import { SheetMaker } from "./controllers/sheetMaker.js";
export class Excel {
    constructor(excelRowContainer, excelRowNumber, excelColNumber, excelsHandler) {
        this.activeSheetIndex = 0;
        this.sheetsArr = [];
        this.excelRowContainer = excelRowContainer;
        this.excelRowNumber = excelRowNumber;
        this.excelColNumber = excelColNumber;
        this.excelsHandler = excelsHandler;
        this.init();
    }
    init() {
        this.createExcelElement();
        this.createInitialSheet();
        this.appendChildrenToExcelElement();
        this.updateContentArea();
        this.notifyExcelsHandler();
        this.handleEvents();
    }
    createExcelElement() {
        this.excelElement = document.createElement("div");
        this.excelElement.className = "excel resizable";
        this.excelElement.id = `row-${this.excelRowNumber}-col-${this.excelColNumber}`;
        this.excelElement.role = "gridcell";
        this.excelElement.ariaRowIndex = `${this.excelRowNumber}`;
        this.excelElement.ariaColIndex = `${this.excelColNumber}`;
        this.excelRowContainer.appendChild(this.excelElement);
    }
    createInitialSheet() {
        const sheetName = "Sheet1";
        this.sheetsArr.push({
            name: sheetName,
            instance: new SheetMaker(sheetName, this.excelRowNumber, this.excelColNumber, 0),
        });
    }
    appendChildrenToExcelElement() {
        this.excelElement.innerHTML = "";
        this.contentArea = document.createElement("div");
        this.contentArea.className = "content-area";
        const sheetBar = this.createSheetBar();
        this.excelElement.append(this.contentArea, sheetBar);
    }
    updateContentArea() {
        this.contentArea.innerHTML = "";
        const activeSheet = this.sheetsArr[this.activeSheetIndex].instance;
        // todo : redefine top and middle section
        this.contentArea.appendChild(activeSheet.elements.topSection);
        this.contentArea.appendChild(activeSheet.elements.middleSection);
    }
    createSheetBar() {
        const sheetBar = document.createElement("div");
        sheetBar.className = "sheet-bar";
        const controls = document.createElement("div");
        controls.className = "sheet-controls";
        controls.innerHTML = `<button class="add-sheet">+</button>`;
        const tabs = document.createElement("div");
        tabs.className = "sheet-tabs";
        this.updateSheetTabs(tabs);
        const scroll = document.createElement("div");
        scroll.className = "sheet-scroll";
        scroll.innerHTML = `
            <button class="scroll-left">◀</button>
            <button class="scroll-right">▶</button>
        `;
        sheetBar.append(controls, tabs, scroll);
        const addSheetButton = controls.querySelector(".add-sheet");
        addSheetButton.addEventListener("click", () => this.addSheet());
        tabs.addEventListener("click", (e) => this.handleSheetTabClick(e));
        return sheetBar;
    }
    handleSheetTabClick(e) {
        var _a, _b;
        const target = e.target;
        if (target.classList.contains("sheet-tab")) {
            const index = parseInt((_a = target.dataset.index) !== null && _a !== void 0 ? _a : "0");
            this.switchSheet(index);
        }
        else if (target.classList.contains("close-tab")) {
            const index = parseInt((_b = target.dataset.index) !== null && _b !== void 0 ? _b : "0");
            this.removeSheet(index);
        }
    }
    updateSheetTabs(tabsContainer) {
        tabsContainer.innerHTML = this.sheetsArr
            .map((sheet, index) => `
            <div class="sheet-tab ${index === this.activeSheetIndex ? "active" : ""}" data-index="${index}">
                ${sheet.name}
                <button class="close-tab" data-index="${index}">✖</button>
            </div>
        `)
            .join("");
    }
    addSheet() {
        const newIndex = this.sheetsArr.length;
        // todo : check if the name already exists
        const newName = `Sheet${newIndex + 1}`;
        this.sheetsArr.push({
            name: newName,
            instance: new SheetMaker(newName, this.excelRowNumber, this.excelColNumber, newIndex),
        });
        this.switchSheet(newIndex);
        this.updateSheetTabs(this.excelElement.querySelector(".sheet-tabs"));
    }
    switchSheet(index) {
        if (index !== this.activeSheetIndex &&
            index >= 0 &&
            index < this.sheetsArr.length) {
            this.activeSheetIndex = index;
            this.updateContentArea();
            this.updateSheetTabs(this.excelElement.querySelector(".sheet-tabs"));
        }
    }
    removeSheet(index) {
        if (this.sheetsArr.length <= 1) {
            alert("You cannot remove the last sheet.");
            return;
        }
        this.sheetsArr.splice(index, 1);
        if (index === this.activeSheetIndex) {
            this.activeSheetIndex = Math.max(0, index - 1);
        }
        else if (index < this.activeSheetIndex) {
            this.activeSheetIndex--;
        }
        this.updateContentArea();
        this.updateSheetTabs(this.excelElement.querySelector(".sheet-tabs"));
    }
    handleEvents() {
        this.excelElement.addEventListener("click", (e) => {
            this.handleMouseDown(e);
        });
    }
    handleMouseDown(e) {
        e.preventDefault();
        this.notifyExcelsHandler();
    }
    notifyExcelsHandler() {
        this.excelsHandler.updateCurrExcel(this.excelRowNumber, this.excelColNumber, this.sheetsArr[this.activeSheetIndex]);
    }
}
