export class RibbonFunctionalities {
    constructor(rumblesheetElement) {
        this.rumblesheetElement = rumblesheetElement;
        this.initializeElements();
        this.updateRibbonScrollBtnsVisible();
    }
    initializeElements() {
        this.tabButtons = document.querySelectorAll(".tablist-items");
        this.tabContents = document.querySelectorAll(".focus-tab");
        this.toggleContentBtn = document.getElementById("toggle-content");
        this.focusZone = document.querySelector(".focus-zone");
        this.featureMenuBtns = document.querySelectorAll(".feature-menu");
        this.ribbon = document.getElementById("ribbon");
        this.scrollLeftBtn = document.querySelector(".scroll-left");
        this.scrollRightBtn = document.querySelector(".scroll-right");
        this.focusContent = document.querySelector(".focus-content");
    }
    toggleTabForTarget(target) {
        const index = Array.from(this.tabButtons).indexOf(target);
        this.tabButtons.forEach((tab) => {
            tab.classList.remove("active");
            tab.classList.remove("leftToActive");
            tab.classList.remove("rightToActive");
        });
        this.tabContents.forEach((content) => content.classList.remove("active"));
        this.tabButtons[index].classList.add("active");
        this.tabContents[index].classList.add("active");
        if (this.tabButtons[index - 1]) {
            this.tabButtons[index - 1].classList.add("leftToActive");
        }
        if (this.tabButtons[index + 1]) {
            this.tabButtons[index + 1].classList.add("rightToActive");
        }
    }
    toggleContentBtnFunc() {
        if (this.focusZone) {
            this.focusZone.classList.toggle("expanded");
        }
        this.updateRibbonScrollBtnsVisible();
    }
    handleFeatureMenuToggle(event) {
        const target = event.target;
        const tabSection = target.closest(".tab-section");
        const correspondingMenuBox = tabSection.querySelector(".feature-box");
        if (!this.ribbon || !this.focusZone || !this.rumblesheetElement)
            return;
        const ribbonRect = this.ribbon.getBoundingClientRect();
        const offset = 5;
        const relativeX = event.clientX - ribbonRect.left + offset - this.focusZone.offsetLeft;
        const relativeY = event.clientY - ribbonRect.top + offset - this.focusZone.offsetTop;
        correspondingMenuBox.classList.toggle("opened");
        let adjustedX = relativeX;
        let adjustedY = relativeY;
        const fromRightBottomOffset = 25;
        if (correspondingMenuBox.offsetWidth + relativeX >=
            this.rumblesheetElement.clientWidth) {
            adjustedX =
                relativeX -
                    (correspondingMenuBox.offsetWidth +
                        relativeX -
                        this.rumblesheetElement.clientWidth) -
                    fromRightBottomOffset;
        }
        if (correspondingMenuBox.offsetHeight + relativeY >=
            this.rumblesheetElement.clientHeight) {
            adjustedY =
                relativeY -
                    (correspondingMenuBox.offsetHeight +
                        relativeY -
                        this.rumblesheetElement.clientHeight) -
                    fromRightBottomOffset;
        }
        correspondingMenuBox.style.left = `${adjustedX}px`;
        correspondingMenuBox.style.top = `${adjustedY}px`;
        if (correspondingMenuBox.classList.contains("opened")) {
            const handleOutsideClick = (e) => {
                if (!correspondingMenuBox.contains(e.target) &&
                    e.target !== target) {
                    correspondingMenuBox.classList.remove("opened");
                    document.removeEventListener("click", handleOutsideClick);
                }
            };
            document.addEventListener("click", handleOutsideClick);
        }
    }
    handleScrollRibbon(event) {
        const content = document.querySelector(".focus-tab.active");
        if (event.target instanceof HTMLElement) {
            if (event.target.closest(".scroll-left")) {
                content.scrollLeft -= 150;
            }
            else if (event.target.closest(".scroll-right")) {
                content.scrollLeft += 150;
            }
        }
    }
    isOverflowingHorizontally(container) {
        return container.scrollWidth > container.clientWidth;
    }
    updateRibbonScrollBtnsVisible() {
        if (!this.focusContent || !this.scrollLeftBtn || !this.scrollRightBtn)
            return;
        const content = document.querySelector(".focus-tab.active");
        if (this.isOverflowingHorizontally(content)) {
            if (!content.classList.contains("scrollAvailable")) {
                this.focusContent.classList.add("scrollAvailable");
                this.scrollLeftBtn.classList.add("scrollAvailable");
                this.scrollRightBtn.classList.add("scrollAvailable");
            }
        }
        else {
            this.focusContent.classList.remove("scrollAvailable");
            this.scrollLeftBtn.classList.remove("scrollAvailable");
            this.scrollRightBtn.classList.remove("scrollAvailable");
        }
    }
}
