import { EventManager } from "./core/eventManager.js";
import { ExcelsHandler } from "./core/excelsHandler.js";
import { Ribbon } from "./ribbon/ribbon.js";
export class Rumblesheet {
    constructor(wrapperContainerId) {
        document.addEventListener("DOMContentLoaded", () => {
            const wrapperContainer = document.getElementById(wrapperContainerId);
            if (!wrapperContainer) {
                throw new Error(`Rumblesheet container with ID '${wrapperContainer}' not found.`);
            }
            this.wrapperContainer = wrapperContainer;
            this.init();
        });
    }
    init() {
        const rumblesheetContainer = this.createRumblesheetContainer();
        const ribbonContainer = this.createRibbonContainer(rumblesheetContainer);
        const mainContainer = this.createMainContainer(rumblesheetContainer);
        this.wrapperContainer.appendChild(rumblesheetContainer);
        this.initializeInstances(ribbonContainer, mainContainer);
    }
    createRumblesheetContainer() {
        const rumblesheetContainer = document.createElement("div");
        rumblesheetContainer.id = "rumblesheet";
        rumblesheetContainer.className = "rumblesheet";
        return rumblesheetContainer;
    }
    createRibbonContainer(parent) {
        const ribbonContainer = document.createElement("div");
        ribbonContainer.id = "ribbon";
        ribbonContainer.className = "ribbon";
        parent.appendChild(ribbonContainer);
        return ribbonContainer;
    }
    createMainContainer(parent) {
        const mainContainer = document.createElement("div");
        mainContainer.id = "mainContainer";
        mainContainer.className = "mainContainer";
        parent.appendChild(mainContainer);
        return mainContainer;
    }
    initializeInstances(ribbonContainer, mainContainer) {
        const maxRow = 3;
        const maxCol = 3;
        this.ribbon = new Ribbon(ribbonContainer, ribbonContainer.parentElement);
        this.excelHandler = new ExcelsHandler(mainContainer, maxRow, maxCol);
        this.eventManager = new EventManager(this.excelHandler, this.ribbon);
    }
}
